/**
 * {标签}
 * @author chensbox<chensbox@foxmail.com>
 * @date 2023/04/28 16:14:10
 */

import { css, html, Component, styleMap } from '@bd/core'
import '../icon/index.js'

const ANIMATION = {
  duration: 100,
  custom: [{ transform: 'scaleX(0)' }, { transform: 'scaleX(1)' }]
}

class Tag extends Component {
  static props = {
    type: {
      type: String,
      default: 'default'
    },
    closable: {
      type: Boolean,
      default: false
    },
    hit: {
      type: Boolean,
      default: false
    },
    color: '',
    size: {
      type: String,
      default: 'xl'
    },
    effect: {
      type: String,
      default: 'light' // dark / light / plain
    }
  }

  static styles = [
    css`
      :host {
        position: relative;
        display: inline-block;
        text-align: center;
        box-sizing: border-box;
      }

      .tag {
        display: inline-block;
        box-sizing: border-box;
        width: 100%;
        font-size: 12px;
        border-radius: 4px;
        white-space: nowrap;
      }
      .close-btn {
        padding: 4px;
        display: inline-block;
        line-height: 0;
        border-radius: 50%;
        vertical-align: middle;
        &:hover {
          background: pink;
          cursor: pointer;
        }
      }
    `,
    //尺寸
    css`
      @use 'sass:map';
      $sizes: (
        s: (
          h: 20px,
          p: 0 5px,
          s: 0.7
        ),
        m: (
          h: 24px,
          p: 0 8px,
          s: 0.8
        ),
        l: (
          h: 28px,
          p: 0 10px,
          s: 0.8
        ),
        xl: (
          h: 32px,
          p: 0 10px,
          s: 1
        )
      );

      @loop $s, $v in $sizes {
        :host([size='#{$s}']) {
          height: map.get($v, 'h');
          .tag {
            padding: map.get($v, 'p');
            line-height: calc(map.get($v, 'h') - 2px);
          }
          .close-btn {
            --size: 10px;
            transform: scale(map.get($v, 's'));
          }
        }
      }
    `,
    // 配色
    css`
      $colors: (
        default: 'plain',
        info: 'grey',
        success: 'green',
        warning: 'orange',
        danger: 'red'
      );

      @loop $t, $c in $colors {
        :host([type='#{$t}']) {
          &:host([effect='light']) {
            .tag {
              background-color: var(--color-#{$c}-a);
              border: 1px solid transparent;
              @if $t == 'default' {
                color: var(--color-blue-3);
              } @else {
                color: var(--color-#{$c}-3);
              }
            }
            .close-btn:hover {
              color: #fff;
              @if $t == 'default' {
                background-color: var(--color-blue-2);
              } @else {
                background-color: var(--color-#{$c}-2);
              }
            }
          }

          &:host([effect='dark']) {
            .tag {
              @if $t == 'default' {
                background-color: var(--color-blue-3);
                border: 1px solid var(--color-blue-3);
              } @else {
                background-color: var(--color-#{$c}-3);
                border: 1px solid var(--color-#{$c}-3);
              }
              color: #fff;
            }
            .close-btn:hover {
              @if $t == 'default' {
                background-color: var(--color-blue-1);
              } @else {
                background-color: var(--color-#{$c}-1);
              }
            }
          }

          &:host([effect='plain']) {
            .tag {
              @if $t == 'default' {
                color: var(--color-blue-3);
              } @else {
                color: var(--color-#{$c}-3);
              }
              background-color: #fff;
              border: 1px solid var(--color-#{$c}-a);
            }
            .close-btn:hover {
              color: #fff;
              @if $t == 'default' {
                background-color: var(--color-blue-1);
              } @else {
                background-color: var(--color-#{$c}-1);
              }
            }
          }

          &:host([hit]) .tag {
            @if $t == 'default' {
              border: 1px solid var(--color-blue-3);
            } @else {
              border: 1px solid var(--color-#{$c}-3);
            }
          }
        }
      }
    `
  ]

  handleClose() {
    this.$refs.tag.$animate(true).then(() => this.$emit('close'))
  }

  render() {
    return html`
      <span
        class="tag"
        #animation=${ANIMATION}
        ref="tag"
        style=${styleMap({ 'background-color': this.color })}
      >
        <slot></slot>
        ${this.closable
          ? html`<div class="close-btn" @click=${this.handleClose}>
              <wc-icon name="close"></wc-icon>
            </div>`
          : ''}
      </span>
    `
  }
}

Tag.reg('tag')
